#!/usr/bin/env python
#
# Migrates an existing passwd file for opscenter into the new opscenter sqlite db.
#

import sys
import platform
version = platform.python_version()
short_version = float(version[:version.index('.', 2)])
if short_version < 2.6:
    print """
    This script is using an unsupported version of python (%s). It requires python2.6 or
    python2.7. If your system is defaulting to an unsupported version
    of python, please call this script directly with a supported version of python.
    For example:

    sudo </path/to/python> migrate_password_file.py <old_password_file_and_path> <new_password_file_and_path>
    """ % (version, )
    sys.exit(1)

from os import path
import sqlite3

usage_text = """Usage:

    migrate_password_file.py <old_password_file_and_path> <new_password_file_and_path>

Running the script multiple times will overwrite data in the database, run with caution
Existing User role will enter new system as unprivileged.
"""

# roles
ADMIN_ROLE = "admin"
USER_ROLE = "user"

# sql
insert_user = 'insert or replace into users (username, password, groupid) values (?,?,?)'
get_group_by_groupname = 'select groupid from user_groups where groupname = ?'


def get_args():
    if len(sys.argv) != 3:
        print usage_text
        sys.exit(1)

    old_path = sys.argv[1]
    if not path.isfile(old_path):
        print "Cannot find the old password file at %s" % (old_path, )
        print usage_text
        sys.exit(1)

    new_path = sys.argv[2]
    if not path.isfile(new_path):
        print "Cannot find the new password file at %s. Note: the first time opscenter is run with" \
              "authentication enabled one will be created." % (new_path, )
        print usage_text
        sys.exit(1)

    return old_path, new_path


def get_db(new_path):
    try:
        conn = sqlite3.connect(new_path)
    except sqlite3.Error, e:
        print "Error connecting to %s : %s" % (new_path, e)
        sys.exit(1)

    return conn


def get_existing_users(old_path):
    with open(old_path, 'r') as old_password_file:
        for users in old_password_file:
            yield users.rsplit(':')


def get_admin_user_groups(conn):
    cursor = conn.execute(get_group_by_groupname, (ADMIN_ROLE,))
    admin_group_id = cursor.fetchone()[0]
    cursor = conn.execute(get_group_by_groupname, (USER_ROLE,))
    user_group_id = cursor.fetchone()[0]
    return admin_group_id, user_group_id


def put_users_in_db(old_path, conn):
    try:
        admin_group_id, user_group_id = get_admin_user_groups(conn)
    except:
        print "Error retrieving the default roles from the database.  Please first run opscenter with Auth enabled."
        print "Then please create a 'user' role, this role is required for migration."
        sys.exit(1)

    for username, password_hash, role_raw in get_existing_users(old_path):
        role = role_raw[:-1]  # trailing new line

        if role == ADMIN_ROLE:
            conn.execute(insert_user, (username, password_hash, admin_group_id,))
            print "Transferred admin %s" % (username, )
        elif role == USER_ROLE:
            conn.execute(insert_user, (username, password_hash, user_group_id,))
            print "Transferred user %s" % (username, )
        else:
            print "Unrecognized role %s for user %s, skipping that user." % (role, username)


if __name__ == "__main__":
    old_path, new_path = get_args()
    conn = get_db(new_path)
    try:
        with conn:
            put_users_in_db(old_path, conn)
    except sqlite3.Error, e:
        print "Error transferring data %s" % (e, )

    conn.close()

    print "Migration completed."
