#!/usr/bin/env python
# Setup the OpsCenter daemon.
#
# Usage: python setup.py
#
# This script serves two purposes. First, it creates the ssl keys
# and certificates that OpsCenter uses for communication with the
# agents. It also builds a zip/tar containing the necessary agent
# files so that agents can be installed manually if that is desired.
#
# This needs to be run before OpsCenter is started.
#

import platform
import sys
import subprocess
from os import path, mkdir, walk, chdir
from shutil import rmtree, copytree
from zipfile import ZipFile
import tarfile

bin_dir, _ = path.split(sys.argv[0])
run_dir = path.join(bin_dir, "..")
chdir(run_dir)

# Appropriate system specific paths
key_pair_exe = path.abspath(path.join("bin", "create-key-pair"))
keystore_exe = path.abspath(path.join("bin", "create-keystore"))
extract_exe = path.abspath(path.join("bin", "extract-key-from-keystore"))
opsc_cert_dir = "ssl"
agent_cert_dir = path.join("agent", "ssl")
opsc_prefix = path.join(opsc_cert_dir, "opscenter")
agent_prefix = path.join(opsc_cert_dir, "agent")
keystore_location = path.join(opsc_cert_dir, "agentKeyStore")
pem_file = path.join(opsc_cert_dir, "opscenter.pem")
sslconfig = path.join("conf", "ssl.conf")

if not path.exists(opsc_cert_dir):
    mkdir(opsc_cert_dir)
if path.exists(agent_cert_dir):
    rmtree(agent_cert_dir)

# Windows: add .bat to ssl scripts
if platform.system().lower() == "windows":
    key_pair_exe += ".bat"
    keystore_exe += ".bat"
    extract_exe += ".bat"

# Create ssl files
p1 = subprocess.Popen([key_pair_exe, opsc_prefix, sslconfig],
        stderr=subprocess.PIPE, stdout=subprocess.PIPE)

if p1.wait() != 0:
    print "Error when creating opscenter SSL files. Error was:\n"
    stdout, stderr = p1.communicate()
    print 'Stdout: ', stdout
    print 'Stderr: ', stderr
    sys.exit(1)

p2 = subprocess.Popen([keystore_exe, agent_prefix, pem_file],
        stderr=subprocess.PIPE, stdout=subprocess.PIPE)

if p2.wait() != 0:
    print "Error when creating agent keystore. Error was:\n"
    stdout, stderr = p2.communicate()
    print stdout
    print stderr
    sys.exit(1)

p3 = subprocess.Popen([extract_exe, keystore_location],
        stderr=subprocess.PIPE, stdout=subprocess.PIPE)

if p3.wait() != 0:
    print "Error when extracting key from agent keystore. Error was:\n"
    stdout, stderr = p3.communicate()
    print stdout
    print stderr
    sys.exit(1)

copytree(opsc_cert_dir, agent_cert_dir)

# Build tar and zip
try:
    zip_file = ZipFile('agent.zip', 'w')
    for root, _, files in walk('agent'):
        for f in files:
            zip_file.write(path.join(root, f))
finally:
    zip_file.close()

try:
    tar_file = tarfile.open('agent.tar.gz', 'w:gz')
    for root, _, files in walk('agent'):
        for f in files:
            tar_file.add(path.join(root, f))
finally:
    tar_file.close()
