from twisted.internet.ssl import ClientContextFactory
import ssl


class SSLVerifyException(Exception):
    pass


class CassandraClientContextFactory(ClientContextFactory):
    def __init__(self, ca_certs=None, cert_reqs=None, keyfile=None, certfile=None, ssl_version=ssl.PROTOCOL_TLSv1):
        self.ca_certs = ca_certs
        self.cert_reqs = cert_reqs
        self.keyfile = keyfile
        self.certfile = certfile
        self.method = self.builtinSslProtocolToOpenSslMethod(ssl_version) # ClientContextFactory's interface requirement

    @classmethod
    def builtinSslProtocolToOpenSslMethod(cls, builint_proto):
        from OpenSSL import SSL
        lookup_table = {
            ssl.PROTOCOL_SSLv3: SSL.SSLv3_METHOD,
            ssl.PROTOCOL_SSLv23: SSL.SSLv23_METHOD,
            ssl.PROTOCOL_TLSv1: SSL.TLSv1_METHOD,
            # TLSv1.1 & 1.2 require newer version of pyOpenSSL
            # ssl.PROTOCOL_TLSv1_1: SSL.TLSv1_1_METHOD,
            # ssl.PROTOCOL_TLSv1_2: SSL.TLSv1_2_METHOD
        }

        return lookup_table[builint_proto]

    def _ssl_simple_verifyCB(self, conn, x509, errnum, errdepth, ok):
        verified = bool(ok)
        if not verified:
            log.warn("Unable to verify ssl certificate.")
            raise SSLVerifyException("SSL certificate invalid")
        return verified

    def getContext(self, *args, **kwargs):
        from OpenSSL import SSL
        ctx = ClientContextFactory.getContext(self)

        if self.cert_reqs != ssl.CERT_NONE:
            ctx.set_verify(SSL.VERIFY_PEER | SSL.VERIFY_FAIL_IF_NO_PEER_CERT, self._ssl_simple_verifyCB)

        ctx.load_verify_locations(self.ca_certs)
        if self.certfile:
            ctx.use_certificate_file(self.certfile)
        if self.keyfile:
            ctx.use_privatekey_file(self.keyfile)

        return ctx
